@@ -63,6 +63,9 @@ debListParser::debListParser(FileFd *File) :
       forceEssential.emplace_back("apt");
    forceImportant = _config->FindVector("pkgCacheGen::ForceImportant");
    myArch = _config->Find("APT::Architecture");
+   // Possible values are: "all", "native", "installed" and "none"
+   // The "installed" mode is handled by ParseStatus(), See #544481 and friends.
+   string const static essential = _config->Find("pkgCacheGen::Essential");
 }
 									/*}}}*/
 // ListParser::Package - Return the package name			/*{{{*/
@@ -120,9 +123,9 @@ unsigned char debListParser::ParseMultiArch(bool const showErrors)	/*{{{*/
 {
    unsigned char MA;
    auto const MultiArch = Section.Find(pkgTagSection::Key::Multi_Arch);
-   if (MultiArch.empty() == true || MultiArch == "no")
+   if (MultiArch.empty() == true || MultiArch == "no"sv)
       MA = pkgCache::Version::No;
-   else if (MultiArch == "same") {
+   else if (MultiArch == "same"sv) {
       if (ArchitectureAll() == true)
       {
 	 if (showErrors == true)
@@ -133,9 +136,9 @@ unsigned char debListParser::ParseMultiArch(bool const showErrors)	/*{{{*/
       else
 	 MA = pkgCache::Version::Same;
    }
-   else if (MultiArch == "foreign")
+   else if (MultiArch == "foreign"sv)
       MA = pkgCache::Version::Foreign;
-   else if (MultiArch == "allowed")
+   else if (MultiArch == "allowed"sv)
       MA = pkgCache::Version::Allowed;
    else
    {
@@ -169,8 +172,8 @@ bool debListParser::NewVersion(pkgCache::VerIterator &Ver)
    pkgCache::GrpIterator G = Ver.ParentPkg().Group();
 
    // Setup the defaults
-   Ver->SourcePkgName = G->Name;
-   Ver->SourceVerStr = Ver->VerStr;
+   Ver.SourceVersion()->Group = G.MapPointer();
+   Ver.SourceVersion()->VerStr = Ver->VerStr;
 
    // Parse the name and version str
    if (Section.Find(pkgTagSection::Key::Source,Start,Stop) == true)
@@ -191,7 +194,7 @@ bool debListParser::NewVersion(pkgCache::VerIterator &Ver)
 	       {
 		  map_stringitem_t const idx = StoreString(pkgCacheGenerator::VERSIONNUMBER, version);
 		  G = Ver.ParentPkg().Group();
-		  Ver->SourceVerStr = idx;
+		  Ver.SourceVersion()->VerStr = idx;
 	       }
 	    }
 	 }
@@ -208,10 +211,13 @@ bool debListParser::NewVersion(pkgCache::VerIterator &Ver)
    }
 
    // Link into by source package group.
-   Ver->SourcePkgName = G->Name;
+   Ver.SourceVersion()->Group = G.MapPointer();
    Ver->NextInSource = G->VersionsInSource;
    G->VersionsInSource = Ver.MapPointer();
 
+   if (auto ArchVar = Section.Find(pkgTagSection::Key::Architecture_Variant); not ArchVar.empty())
+      Ver.ArchVariant(StoreString(pkgCacheGenerator::MIXED, ArchVar));
+
    Ver->MultiArch = ParseMultiArch(true);
    // Archive Size
    Ver->Size = Section.FindULL(pkgTagSection::Key::Size);
@@ -293,11 +299,7 @@ std::string_view debListParser::Description_md5()
 bool debListParser::UsePackage(pkgCache::PkgIterator &Pkg,
 			       pkgCache::VerIterator &Ver)
 {
-   string const static myArch = _config->Find("APT::Architecture");
-   // Possible values are: "all", "native", "installed" and "none"
-   // The "installed" mode is handled by ParseStatus(), See #544481 and friends.
-   string const static essential = _config->Find("pkgCacheGen::Essential", "all");
-   if (essential == "all" ||
+   if (essential == "all" || essential.empty() ||
        (essential == "native" && Pkg->Arch != 0 && myArch == Pkg.Arch()))
       if (Section.FindFlag(pkgTagSection::Key::Essential,Pkg->Flags,pkgCache::Flag::Essential) == false)
 	 return false;
@@ -309,7 +311,7 @@ bool debListParser::UsePackage(pkgCache::PkgIterator &Pkg,
    if (std::find(forceEssential.begin(), forceEssential.end(), Pkg.Name()) != forceEssential.end())
    {
       if ((essential == "native" && Pkg->Arch != 0 && myArch == Pkg.Arch()) ||
-	  essential == "all")
+	  essential == "all" || essential.empty())
 	 Pkg->Flags |= pkgCache::Flag::Essential | pkgCache::Flag::Important;
       else
 	 Pkg->Flags |= pkgCache::Flag::Important;
@@ -400,10 +402,8 @@ bool debStatusListParser::ParseStatus(pkgCache::PkgIterator &Pkg,
    if (Section.Find(pkgTagSection::Key::Status,Start,Stop) == false)
       return true;
 
-   // UsePackage() is responsible for setting the flag in the default case
-   bool const static essential = _config->Find("pkgCacheGen::Essential", "") == "installed";
-   if (essential == true &&
-       Section.FindFlag(pkgTagSection::Key::Essential,Pkg->Flags,pkgCache::Flag::Essential) == false)
+   if (essential == "installed" &&
+       Section.FindFlag(pkgTagSection::Key::Essential, Pkg->Flags, pkgCache::Flag::Essential) == false)
       return false;
 
    // Isolate the first word
@@ -563,24 +563,6 @@ const char *debListParser::ParseDepends(const char *Start,const char *Stop,
 
    return res;
 }
-const char *debListParser::ParseDepends(const char *Start,const char *Stop,
-					APT::StringView &Package,APT::StringView &Ver,
-					unsigned int &Op, bool ParseArchFlags,
-					bool const StripMultiArch,
-					bool const ParseRestrictionsList,
-				        string Arch)
-{
-   string_view PackageView;
-   string_view VerView;
-
-   auto res = ParseDepends(Start, Stop, PackageView, VerView, Op, (bool)ParseArchFlags,
-   (bool) StripMultiArch, (bool) ParseRestrictionsList, Arch);
-   Package = PackageView;
-   Ver = VerView;
-
-   return res;
-}
-
 
 const char *debListParser::ParseDepends(const char *Start, const char *Stop,
 					string_view &Package, string_view &Ver,
@@ -911,7 +893,7 @@ bool debListParser::ParseProvides(pkgCache::VerIterator &Ver)
 
       do
       {
-	 Start = ParseDepends(Start,Stop,Package,Version,Op, false, false, false);
+	 Start = ParseDepends(Start, Stop, Package, Version, Op, false, false, false, myArch);
 	 const size_t archfound = Package.rfind(':');
 	 if (Start == 0)
 	    return _error->Error("Problem parsing Provides line of %s:%s=%s", Ver.ParentPkg().Name(), Ver.Arch(), Ver.VerStr());
@@ -1038,6 +1020,9 @@ bool debListParser::SameVersion(uint32_t Hash,		/*{{{*/
    unsigned char MultiArch = ParseMultiArch(false);
    if (MultiArch != Ver->MultiArch)
       return false;
+
+   if (ArchVariant() != Ver.ArchVariant())
+      return false;
    // for all practical proposes (we can check): same version
    return true;
 }
